########## MCMC code #############

library(MCMCpack)
library(mvtnorm)
library(BayesLogit)
library(gtools)
library(gdata)
library(msm)

## call the functions 

  mat.sqrt<-function(A)         #square root of a matrix
  {
    ei<-eigen(A)
    d<-ei$values
    d<-(d+abs(d))/2
    d2<-sqrt(d)
    ans<-ei$vectors %*% diag(d2) %*% t(ei$vectors)
    return(ans)
  }
  
  
  mat.sqrt.inv<-function(A)   #square root inverse of a matrix
  {
    ei<-eigen(A)
    d<-ei$values
    d<-(d+abs(d))/2
    d2<-1 / sqrt(d)
    d2[d == 0]<-0
    ans<-ei$vectors %*% diag(d2) %*% t(ei$vectors)
    return(ans)
  }
  
  copy.upper.lower <- function(m) 
  {
    m[lower.tri(m)] <- t(m)[lower.tri(m)]
    m
   }

Initialize <-
function(Data.list,X.list,
         lambda.mean,lambda.var,
           eta.mean,eta.var,
           m.mean,m.var,
           epsilon_phi = 0.5, cov_tune = 1e-3,seed=12345) {

     set.seed(seed)
  
  G <- sapply(Data.list,nrow)[1]  # number of genes
  N <- sapply(Data.list,ncol)    # number of samples
  K <- length(Data.list) # number of studies
  
  libsize <- sapply(Data.list,colSums)
  logT <- lapply(libsize,log)
  
  v.alpha <- v.beta <- v.phi <- K+1
  
  #1. pi    
  pi.init<-runif(1,0,1)
  
  #2. delta   
  delta.init<-rep(0,K*G)  
  for (i in 1:G){
    delta.init[((i-1)*K+1):(i*K)]<-as.vector(rbinom(n=K,size=1,prob=pi.init))
  }
  
  #3. corr
  rho.cov<-rwish(v=v.beta,S=diag(1,nrow=K,ncol=K))
  rho.cor<-mat.sqrt.inv(diag(x=diag(rho.cov))) %*% rho.cov %*% mat.sqrt.inv(diag(x=diag(rho.cov)))
  rho.init.de<-rho.init.non.de<-rho.cor[which(upper.tri(rho.cor)=='TRUE')]
  
  r.cov<-rwish(v=v.alpha,S=diag(1,nrow=K,ncol=K))
  r.cor<-mat.sqrt.inv(diag(x=diag(r.cov))) %*% r.cov %*% mat.sqrt.inv(diag(x=diag(r.cov)))
  r.init<-r.cor[which(upper.tri(r.cor)=='TRUE')]
  
  t.cov<-rwish(v=v.phi,S=diag(1,nrow=K,ncol=K))
  t.cor<-mat.sqrt.inv(diag(x=diag(t.cov))) %*% t.cov %*% mat.sqrt.inv(diag(x=diag(t.cov)))
  t.init<-t.cor[which(upper.tri(t.cor)=='TRUE')]
  
  #4. sigma2
  sigma2.de.init<- sigma2.non.de.init<- tau2.init <- xi2.init <- rep(1,K)

  #5. grand mean
  lambda.init<-rnorm(G,mean=lambda.mean,sd=sqrt(lambda.var))
  eta.init<-rnorm(G,mean=eta.mean,sd=sqrt(eta.var))
  m.init<-rep(m.mean, G)

  #6. main model
  beta.init<-rep(0,K*G)
  alpha.init<-rep(0,K*G)
  logphi.init<-rep(0,K*G)
  
  for (g in 1:G) {  
    Cov.beta<- mat.sqrt(diag(sigma2.non.de.init))%*% rho.cor %*% 
      mat.sqrt(diag(sigma2.non.de.init))
    Cov.alpha<- mat.sqrt(diag(tau2.init))%*% r.cor %*% 
      mat.sqrt(diag(tau2.init))
    Cov.logphi<- mat.sqrt(diag(xi2.init))%*% t.cor %*% 
      mat.sqrt(diag(xi2.init))
    
    beta.init[((g-1)*K+1):(g*K)]<-as.vector( rmvnorm(n=1,
                                                     mean=delta.init[((g-1)*K+1):(g*K)]*lambda.init[g],
                                                     sigma=Cov.beta ) )
    alpha.init[((g-1)*K+1):(g*K)]<-as.vector( rmvnorm(n=1,
                                                      mean=rep(eta.init[g],K),
                                                      sigma=Cov.alpha  ) )
    logphi.init[((g-1)*K+1):(g*K)]<-as.vector( rmvnorm(n=1,
                                                       mean=rep(m.init[g],K),
                                                       sigma=Cov.logphi ) )
  }  
  
  #7. omega
  
  omega.init <- vector('list',length=K)
  
  for (k in 1:K) {
    omega.init[[k]] <- matrix(,nrow=G,ncol=N[k])
  }
  for (k in 1:K) {
    for (g in 1:G) {
      omega.init[[k]][g,]<-rpg(num=N[k],h=as.numeric(Data.list[[k]][g,])+1/exp(m.mean),z=0)
    }  
  }
  
  init=list(Prob=pi.init,
            Delta=delta.init,
            Rho.DE=rho.init.de,
            Rho.Non.DE=rho.init.non.de,
            RR=r.init,
            TT=t.init,
            Sigma2.DE=sigma2.de.init,
            Sigma2.Non.DE=sigma2.non.de.init,
            Tau2=tau2.init,
            Xi2=xi2.init,
            Lambda=lambda.init,
            Eta=eta.init,
            M=m.init,
            Beta=beta.init,
            Alpha=alpha.init,
            LogPhi=logphi.init,
            Omega=omega.init)
  
  print("MCMC chain initialized")
  return(init)
}


Store <-
function(Data.list,X.list,iteration)  {
  
  G <- sapply(Data.list,nrow)[1]  # number of genes
  N <- sapply(Data.list,ncol)    # number of samples
  K <- length(Data.list) # number of studies
  delta <- matrix(rep(0, K*G*iteration),nrow=K*G,ncol=iteration)  
  
  beta<-matrix(rep(0,K*G*iteration),nrow=K*G,ncol=iteration)
    
  store <- list(Delta=delta,
                Beta=beta)
  print("Store matrix is ready")
  return(store)
}


MCMCRun <- 
function(Data.list,X.list,Init.value,Store.value,iteration,
                    lambda.mean,lambda.var,eta.mean,eta.var,
                    m.mean,m.var,
                    epsilon_phi = 0.5, cov_tune = 1e-3,seed=12345) {
   
  G <- sapply(Data.list,nrow)[1]  # number of genes
  N <- sapply(Data.list,ncol)    # number of samples
  K <- length(Data.list) # number of studies
  
  libsize <- lapply(Data.list,colSums)
  logT <- lapply(libsize,log)
  v.alpha <- v.beta <- v.phi <- K+1
  
  #### alpha,beta update function:

  update.alphabeta <- function(y,lib,X.mat,omega,logphi,delta,lambda,eta,sigma2.de,sigma2.non.de,tau2) {
    
    Omega <- diag(omega)
    r <- 1/exp(logphi)
    
    c = c(eta,lambda*delta)
    C = diag(c(tau2,
               ifelse(delta==0,sigma2.non.de,sigma2.de) ) )
    
    V = solve(t(X.mat)%*%Omega%*% X.mat+solve(C)) 
    z = (y-r)/(2*omega) - lib
    m = V%*%(t(X.mat)%*%Omega%*%z + solve(C)%*%c)
    rmvnorm(n=1,mean=m,sigma=V)
  }

  #### logphi update function 
  
  update.logphi<- function(m,Pi,logphi,alpha,beta,X.list,y,lib) {
    
    y.k<-lapply(y,'[',g,,drop=F)
    
    sum.fun<-function(alpha,beta,X,logphi,lib,y){
      sum( dnbinom(y,size=ifelse(is.na(1/exp(logphi)),1/epsilon_phi,1/exp(logphi)),
                   mu=exp(lib+alpha+beta*X), log=T) )  
    }
    
    old_logp<-mapply(sum.fun,alpha=alpha,
                     beta=beta,X=X.list,logphi=logphi,
                     lib=lib,y=y.k)
    
    old_hyper<-ifelse(dmvnorm(logphi,mean=rep(m,K),sigma=Pi+cov_tune*diag(K), log=F)==0,
                      epsilon_phi,
                      dmvnorm(logphi,mean=rep(m,K),sigma=Pi+cov_tune*diag(K), log=T) )
    
    old_log_like  <- sum(old_logp) + old_hyper
        
    new.logphi <- rep(NA,length(logphi))
     for (i in 1:length(logphi)){
       new.logphi[i]<-rtnorm(1,mean=logphi[i],sd=sqrt(diag(Pi)[i]),lower=-20,upper=10)
    }
    
    new_logp<-mapply(sum.fun,alpha=alpha,
                     beta=beta,X=X.list,logphi=new.logphi,
                     lib=lib,y=y.k) 
    
    new_hyper<-ifelse(dmvnorm(new.logphi,mean=rep(m,K),sigma=Pi+cov_tune*diag(K), log=F)==0,
                      epsilon_phi,
                      dmvnorm(new.logphi,mean=rep(m,K),sigma=Pi+cov_tune*diag(K), log=T) )
    
    new_log_like  <- sum(new_logp) +new_hyper
    
    if(!is.na(new_log_like))  {
      if ( !is.na(old_log_like) ) {
        if ( !is.na(new_log_like - old_log_like) ){
          if(new_log_like - old_log_like > log(runif(1)) ) 
            return(new.logphi)
          else  return(logphi)
        }
        else return(logphi)
      }
      else return(new.logphi)
    }
    else return(logphi)
    
  }

  ### update lambda, eta, m 
  
  update.lambda<-function(beta,lambda,delta,Sigma) {
    
    old_logp<-dmvnorm(beta,mean=lambda*delta,sigma=Sigma+cov_tune*diag(K),log=T)
    
    old_hyper<-dnorm(lambda,mean=lambda.mean,sd=sqrt(lambda.var),log=T)
    
    old_log_like  <- old_logp + old_hyper
    
    new.lambda  <- rnorm(1,mean=lambda,sd=sqrt(lambda.var)) # Proposal
    
    new_logp<-dmvnorm(beta,mean=new.lambda*delta,sigma=Sigma+cov_tune*diag(K),log=T)
    
    new_hyper<-dnorm(new.lambda,mean=lambda.mean,sd=sqrt(lambda.var),log=T)
    
    new_log_like  <- new_logp + new_hyper
    
    if(!is.na(new_log_like))  {
      if ( !is.na(old_log_like) ) {
        if ( !is.na(new_log_like - old_log_like) ){
          if(new_log_like - old_log_like > log(runif(1)) ) 
            return(new.lambda)
          else  return(lambda)
        }
        else return(lambda)
      }
      else return(new.lambda)
    }
    else return(lambda)    
  } 
  
  update.eta<-function(alpha,eta,Lambda) {
    
    old_logp<-dmvnorm(alpha,mean=rep(eta,K),sigma=Lambda+cov_tune*diag(K),log=T)
    
    old_hyper<-dnorm(eta,mean=eta.mean,sd=sqrt(eta.var),log=T)
    
    old_log_like  <- old_logp + old_hyper
    
    new.eta  <- rnorm(1,mean=eta,sd=sqrt(eta.var)) # Proposal
    
    new_logp<-dmvnorm(alpha,mean=rep(new.eta,K),sigma=Lambda+cov_tune*diag(K),log=T)
    
    new_hyper<-dnorm(new.eta,mean=eta.mean,sd=sqrt(eta.var),log=T)
    
    new_log_like  <- new_logp + new_hyper
    
    if(!is.na(new_log_like))  {
      if ( !is.na(old_log_like) ) {
        if ( !is.na(new_log_like - old_log_like) ){
          if(new_log_like - old_log_like > log(runif(1)) ) 
            return(new.eta)
          else  return(eta)
        }
        else return(eta)
      }
      else return(new.eta)
    }
    else return(eta) 
  } 
  
  update.m<-function(logphi,m,Pi) {
    
    old_logp<-dmvnorm(logphi,mean=rep(m,K),sigma=Pi+cov_tune*diag(K),log=T)
    
    old_hyper<-dnorm(m,mean=m.mean,sd=sqrt(m.var),log=T)
    
    old_log_like  <- old_logp + old_hyper

    new.m <- rtnorm(1,mean=m, sd=sqrt(m.var),lower = -20, upper=10)
     
    new_logp<-dmvnorm(logphi,mean=rep(new.m,K),sigma=Pi+cov_tune*diag(K),log=T)
    
    new_hyper<-dnorm(new.m,mean=m.mean,sd=sqrt(m.var),log=T)
    
    new_log_like  <- new_logp + new_hyper
    
    if(!is.na(new_log_like))  {
      if ( !is.na(old_log_like) ) {
        if ( !is.na(new_log_like - old_log_like) ){
          if(new_log_like - old_log_like > log(runif(1)) ) 
            return(new.m)
          else  return(m)
        }
        else return(m)
      }
      else return(new.m)
    }
    else return(m) 
  } 
 
 
#### sigma2.beta, sigma2.alpha, sigma2.log.phi update functions
  
  update.sigma2de<-function(beta,lambda,delta){
    rinvgamma(1,shape=sum(delta)/2,
              scale=(1/2)*sum((delta*(beta-lambda*delta)^2)[which(abs(beta)<5)] )   )  
  }
  
  update.sigma2nonde<-function(beta,lambda,delta){
    rinvgamma(1,shape=(G-sum(delta))/2 ,
              scale=(1/2)*sum((beta[which(delta==0 & abs(beta)<5)])^2)   )  
  }
  
  update.tau2<-function(alpha,eta){
    rinvgamma(1,shape=G/2,
              scale=(1/2)*sum((alpha-eta)^2)  )
  }
  
  update.xi2 <-function(logphi,m){
    rinvgamma(1,shape=G/2,
              scale=(1/2)* sum((logphi-m)^2)  )
  }


  ##### rho, r, t update functions
  
  update.rho<-function(beta,lambda,delta) {
    beta.i<-rep(0,K)
    lambda.i<-rep(mean(lambda),K)
    for (k in 1:K){
      beta.i[k]<-mean(beta[seq(k,length(beta),length(beta)/K)])
    }  
    Cov<-riwish(v=v.beta+K, S=diag(K)+matrix(beta.i-lambda.i) %*% t(matrix(beta.i-lambda.i))  )  #sum over G
    # then integrate out variance to get the correlation matrix
    Cor<-diag((x=diag(Cov))^(-1/2)) %*%
      Cov %*% diag((x=diag(Cov))^(-1/2))
    rho.out<-Cor[which(upper.tri(Cor)=='TRUE')]
    return(rho.out)
  }
  
  update.r<-function(alpha,eta) {
    alpha.i<-rep(0,K)
    eta.i<-rep(mean(eta),K)
    for (k in 1:K){
      alpha.i[k]<-mean(alpha[seq(k,K*G,G)])
    }  
    Cov<-riwish(v=v.alpha+K, S=diag(K)+ matrix(alpha.i-eta.i) %*% t(matrix(alpha.i-eta.i)) )   #sum over K
    # then integrate out variance to get the correlation matrix
    Cor<-diag((x=diag(Cov))^(-1/2)) %*%
      Cov %*% diag((x=diag(Cov))^(-1/2))
    r.out<-Cor[which(upper.tri(Cor)=='TRUE')]
    return(r.out)
  }
  
  
  update.t<-function(logphi,m) {
    logphi.i<-rep(0,K)
    m.i<-rep(mean(m),K)
    for (k in 1:K){
      logphi.i[k]<-mean(logphi[seq(k,K*G,G)])
    }  
    Cov<-riwish(v=v.phi+K, S=diag(K)+matrix(logphi.i-m.i) %*% t(matrix(logphi.i-m.i)) )  #sum over K
    # then integrate out variance to get the correlation matrix
    Cor<-diag((x=diag(Cov))^(-1/2)) %*%
      Cov %*% diag((x=diag(Cov))^(-1/2))
    t.out<-Cor[which(upper.tri(Cor)=='TRUE')]
    return(t.out)
  }
  

  ### delta_beta update function
  
  update.deltabeta<- function(old.delta,old.beta,new.beta,
                              sigma2.de.i.k,
                              sigma2.non.de.i.k,y,X,lib,
                              alpha,logphi,lambda) {
    
    new.delta<-1-old.delta 
    
    old_logp= sum( dnbinom(y,size=ifelse(is.na(1/exp(logphi)),1/epsilon_phi,1/exp(logphi)),
                           mu=exp(lib+alpha+old.beta*X), log=T) ) +  
      dnorm(old.beta,mean=ifelse(old.delta==0,0,lambda),
           sd=ifelse(old.delta==0,sqrt(sigma2.non.de.i.k),sqrt(sigma2.de.i.k)),log=T) 
    
    new_logp= sum( dnbinom(y,size=ifelse(is.na(1/exp(logphi)),1/epsilon_phi,1/exp(logphi)),
                           mu=exp(lib+alpha+new.beta*X), log=T) ) +  
      dnorm(new.beta,mean=ifelse(new.delta==0,0,lambda),
               sd=ifelse(new.delta==0,sqrt(sigma2.non.de.i.k),sqrt(sigma2.de.i.k)),log=T) 
    
    old_log_like<-old_logp
    new_log_like<-new_logp
    
    if(!is.na(new_log_like))  {
      if ( !is.na(old_log_like) ) {
        if ( !is.na(new_log_like - old_log_like) ){
          
          if(new_log_like - old_log_like > log(runif(1)))
            return(c(new.delta,new.beta)) 
          else
            return(c(old.delta,old.beta)) 
        }
        else return (c(old.delta,old.beta))
      }
      else return (c(new.delta,new.beta))
    }
    else 
      return (c(old.delta,old.beta))
    
  }    
  

  ## list of library and count: for block update in the last step
  lib.g<-rep(logT,G)
  X.g<-rep(X.list,G)
  y.k.g<-vector('list',K*G)
  for (g in 1:G){
    for (k in 1:K){
      y.k.g[[(g-1)*K+k]]<-Data.list[[k]][g,]
    }
  }
  
  ## calculate the observed ES
  
  es.seq =function (y,X,lib) {
    y=array(y)
    v1=log((y[which(X==1)]+1)/(lib[which(X==1)]))
    v2=log((y[which(X==0)]+1)/(lib[which(X==0)]))
    u=mean(v1-v2)
    return(u)
  }
  
  beta.obs<-matrix(rep(0,G*K),nrow=G,ncol=K) 
  for (g in 1:G){
    for (k in 1:K){
      lib<-libsize[[k]]
      beta.obs[g,k]<-es.seq(y=Data.list[[k]][g,],X=X.list[[k]],lib=lib)
    }
  }


## initialize the chain
  
  pi <- Init.value[["Prob"]]
  delta <- Store.value[["Delta"]][,1] <- Init.value[["Delta"]]
  sigma2.de <- Init.value[["Sigma2.DE"]]
  sigma2.non.de <- Init.value[["Sigma2.Non.DE"]]
  tau2 <- Init.value[["Tau2"]]
  xi2 <- Init.value[["Xi2"]]
  rho.de <- Init.value[["Rho.DE"]]
  rho.non.de <- Init.value[["Rho.Non.DE"]]
  rr <- Init.value[["RR"]]
  tt <- Init.value[["TT"]]
  lambda <- Init.value[["Lambda"]]
  eta <- Init.value[["Eta"]]
  m <- Init.value[["M"]]
  beta <- Store.value[["Beta"]][,1] <- Init.value[["Beta"]]
  alpha <- Init.value[["Alpha"]]
  logphi <- Init.value[["LogPhi"]]
  omega <- Init.value[["Omega"]]

  ### Iteration part
  for (iter in 2:iteration) {   
    
    set.seed(seed+iter)
    
    startTime <- proc.time()
       
    rho.cor.de <- matrix(,ncol=K,nrow=K) 
    rho.cor.non.de <- matrix(,ncol=K,nrow=K)  
    r.cor <- matrix(,ncol=K,nrow=K)
    t.cor <- matrix(,ncol=K,nrow=K)
    
    rho.cor.de[which(upper.tri(rho.cor.de)=='TRUE')]<-rho.de
    rho.cor.non.de[which(upper.tri(rho.cor.non.de)=='TRUE')]<-rho.non.de
    r.cor[which(upper.tri(r.cor)=='TRUE')]<-rr
    t.cor[which(upper.tri(t.cor)=='TRUE')]<-tt
        
    diag(rho.cor.de)<-rep(1,K)
    diag(rho.cor.non.de)<-rep(1,K)
    diag(r.cor)<-rep(1,K)
    diag(t.cor)<-rep(1,K)
    
    rho.cor.de <- copy.upper.lower(rho.cor.de)
    rho.cor.non.de <- copy.upper.lower(rho.cor.non.de)
    r.cor <- copy.upper.lower(r.cor)
    t.cor <- copy.upper.lower(t.cor)

    ### update alpha+beta,logphi,lambda,eta,m 
    
    alpha.cur=rep(0,K*G)
    beta.cur=rep(0,K*G)
    logphi.cur=rep(0,K*G)
    lambda.cur=rep(0,G)
    eta.cur=rep(0,G)
    m.cur= rep(0,G)
    
    Sigma.de<- mat.sqrt(diag(sigma2.de))%*% rho.cor.de %*% 
      mat.sqrt(diag(sigma2.de))
    Sigma.non.de<- mat.sqrt(diag(sigma2.non.de))%*% rho.cor.non.de %*% 
      mat.sqrt(diag(sigma2.non.de))
    Lambda <- mat.sqrt(diag(tau2))%*% r.cor %*% 
      mat.sqrt(diag(tau2))
    Pi <- mat.sqrt(diag(xi2))%*% t.cor %*% 
      mat.sqrt(diag(xi2))
     
    omega.cur  <- omega
    
    for (g in 1:G) {

      delta.g<-delta[((g-1)*K+1):(g*K)]
      beta.g<- beta[((g-1)*K+1):(g*K)] 
      alpha.g<- alpha[((g-1)*K+1):(g*K)] 
      logphi.g<-logphi[((g-1)*K+1):(g*K)]
      lambda.g<-lambda[g]
      eta.g<- eta[g] 
      m.g<-m[g]
      
    for (k in 1:K) {  	
      
      X.mat <- cbind(rep(1,N[[k]]),X.list[[k]])
      
      alphabeta <- update.alphabeta(y=Data.list[[k]][g,],lib=logT[[k]],X.mat=X.mat,omega=omega[[k]][g,],logphi=logphi.g[k],delta=delta.g[k],lambda=lambda.g,eta=eta.g,sigma2.de=sigma2.de[k],sigma2.non.de=sigma2.non.de[k],tau2=tau2[k])  
           if(abs(alphabeta[2])<=10 & abs(alphabeta[1]) <= 30 ) {
          	     alpha.cur[(g-1)*K+k] <- alphabeta[1]
        	     beta.cur[(g-1)*K+k] <- alphabeta[2]
               } else{   
                  alpha.cur[(g-1)*K+k] <- alpha.g[k]
                  beta.cur[(g-1)*K+k] <- beta.g[k]
               } 

        omega.cur[[k]][g,] <- rpg(num=N[[k]],h=Data.list[[k]][g,]+1/exp(logphi.g[k]),z=c( X.mat%*%c(alpha.cur[(g-1)*K+k],beta.cur[(g-1)*K+k]))+ logT[[k]] )
        
      }
      
     logphi.cur[((g-1)*K+1):(g*K)]<-update.logphi(m=m.g,Pi=Pi,logphi=logphi.g,alpha=alpha.cur[((g-1)*K+1):(g*K)],beta=beta.cur[((g-1)*K+1):(g*K)],X.list=X.list,y=Data.list,lib=logT)         
  
           if(sum(delta.g)==0) {

        lambda.cur[g]<-update.lambda(beta=beta.cur[((g-1)*K+1):(g*K)] ,lambda=lambda.g,delta=delta.g,Sigma=Sigma.non.de)	
        
      } else  {     

        lambda.cur[g]<-update.lambda(beta=beta.cur[((g-1)*K+1):(g*K)],lambda=lambda.g,delta=delta.g,Sigma=Sigma.de)	

      }

       eta.cur[g]<-update.eta(alpha=alpha.cur[((g-1)*K+1):(g*K)] ,eta=eta.g,Lambda=Lambda)
       m.cur[g]<-update.m(logphi=logphi.cur[((g-1)*K+1):(g*K)],m=m.g,Pi=Pi)
    
   } ## end of g in 1:G
   
    beta <- beta.cur
    alpha <-alpha.cur
    logphi <-logphi.cur      
    lambda <-lambda.cur  
    eta <-eta.cur
    m <-m.cur
    omega <- omega.cur   
   
    # update sigma2.beta, sigma2.alpha, sigma2.log.phi
    
    sigma2.de.cur<-rep(0,K)
    sigma2.non.de.cur<-rep(0,K)
    tau2.cur<-rep(0,K)
    xi2.cur<-rep(0,K)
    
    for (k in 1:K) {
      
      sigma2.de.cur[k]<-update.sigma2de(
        beta=beta.cur[seq(k,(G-1)*K+k,by=K)],lambda=lambda,delta=delta[seq(k,(G-1)*K+k,by=K)])
      
      sigma2.non.de.cur[k]<-update.sigma2nonde(
        beta=beta.cur[seq(k,(G-1)*K+k,by=K)],lambda=lambda,delta=delta[seq(k,(G-1)*K+k,by=K)])
      
      tau2.cur[k]<-update.tau2(
        alpha=alpha[seq(k,(G-1)*K+k,by=K)],eta=eta)
      
      xi2.cur[k]<-update.xi2(
        logphi=logphi[seq(k,(G-1)*K+k,by=K)],m=m)
    }

    sigma2.de <- sigma2.de.cur
    sigma2.non.de <- sigma2.non.de.cur
    tau2 <- tau2.cur
    xi2 <- xi2.cur

    ### update rho, r, t
    beta.mat<-matrix(beta,byrow=F,nrow=K,ncol=G)
    delta.mat<-matrix(delta,byrow=F,nrow=K,ncol=G)
    delta.sum<-apply(delta.mat,2,sum)
    de.index<-which(delta.sum>=1)
    non.de.index<-which(delta.sum==0)
    beta.de<-c(beta.mat[,de.index])
    beta.non.de<-c(beta.mat[,non.de.index])
    lambda.de<-lambda[de.index]
    lambda.non.de<-lambda[non.de.index]
    
    rho.de <- update.rho(beta=beta.de,lambda=lambda.de)
    rho.non.de <-update.rho(beta=beta.non.de,lambda=lambda.non.de)
    r.cur<-update.r(alpha=alpha,eta=eta)
    t.cur<-update.t(logphi=logphi,m=m)

    ## block update of delta and beta
    
    
    deltabeta.cur<-unlist(mapply(update.deltabeta,
                                 old.delta=delta,old.beta=beta.cur,
                                 new.beta=beta.cur,sigma2.de.i.k =sigma2.de,
                                 sigma2.non.de.i.k=sigma2.non.de,
                                 y=y.k.g,X=X.g,lib=lib.g,
                                 alpha=alpha,logphi=logphi,lambda=rep(lambda,each=K)
    ))

    delta.cur<-deltabeta.cur[1,]
    beta.cur<-deltabeta.cur[2,]
    
    delta <- Store.value[['Delta']][,iter]<- delta.cur
    beta <- Store.value[['Beta']][,iter]<- beta.cur   
    
    endTime <- proc.time()
    
    print(iter)
    print(endTime-startTime) 
    
 } ## end of iteration

    print("MCMC is complete")
    return(Store.value) 

} ## end of MCMCRun function